Package: landsepi
Type: Package
Encoding: UTF-8
Title: Landscape Epidemiology and Evolution
Version: 1.5.3
Date: 2026-01-07
Authors@R: 
    c(person(given = "Loup",
             family = "Rimbaud",
             role = "aut",
             email = "loup.rimbaud@inrae.fr",
             comment = c(ORCID = "0000-0002-8098-9984")),
      person(given = "Marta",
             family = "Zaffaroni",
             role = "aut",
             email = "marta.zaffaroni@inrae.fr",
             comment = c(ORCID = "0000-0002-2951-8626")),
      person(given = "Jean-François",
             family = "Rey",
             role = c("aut","cre"),
             email = "jean-francois.rey@inrae.fr",
             comment = c(ORCID = "0000-0003-3281-6701")),
      person(given = "Julien",
             family = "Papaix",
             role = "aut",
             email = "julien.papaix@inrae.fr"),
      person(given = "Jean-Loup",
             family = "Gaussen",
             role = "ctb",
             email = "jean-loup-thomas.gaussen@inrae.fr"),
      person(given = "Manon",
             family = "Couty",
             role = "ctb",
             email = "manon.couty@insa-lyon.fr"))
Maintainer: Jean-François Rey <jean-francois.rey@inrae.fr>
Description: A stochastic, spatially-explicit, demo-genetic model simulating the spread and evolution 
    of a plant pathogen in a heterogeneous landscape to assess resistance deployment strategies. 
    It is based on a spatial geometry for describing the landscape and allocation of different cultivars, 
    a dispersal kernel for the dissemination of the pathogen, and a SEIR 
    ('Susceptible-Exposed-Infectious-Removed’) structure with a discrete time step.
    It provides a useful tool to assess the performance of a wide range of deployment options with 
    respect to their epidemiological, evolutionary and economic outcomes.
    Loup Rimbaud, Julien Papaïx, Jean-François Rey, Luke G Barrett, 
    Peter H Thrall (2018) <doi:10.1371/journal.pcbi.1006067>.
URL: https://landsepi.pages-forge.inrae.fr/landsepi,
        https://forge.inrae.fr/landsepi/landsepi
BugReports: https://forge.inrae.fr/landsepi/landsepi/-/issues
License: GPL (>= 2) | file LICENSE
LazyData: true
BuildVignettes: true
NeedsCompilation: yes
Biarch: true
SystemRequirements: C++17, gsl
Depends: R (>= 4.2.0), sp (>= 1.0-17)
Imports: methods, utils, stats (>= 3.0.2), grDevices (>= 3.0.0),
        graphics (>= 3.0.0), parallel, Rcpp (>= 0.9.0), Matrix (>=
        1.3-0), mvtnorm, fields, splancs, sf, DBI, RSQLite, foreach,
        doParallel, deSolve
Collate: 'Math-Functions.R' 'RcppExports.R' 'graphics.R' 'AgriLand.R'
        'Class-LandsepiParams.R' 'Cultivars_List.R' 'GPKGTools.R'
        'tools.R' 'Methods-LandsepiParams.R' 'demo_landsepi.R'
        'landsepi-package.R' 'output.R' 'runShiny.R' 'simul_landsepi.R'
LinkingTo: Rcpp, testthat
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), shiny, shinyjs, DT, knitr, rmarkdown
VignetteBuilder: knitr
Packaged: 2026-01-07 08:11:15 UTC; root
Author: Loup Rimbaud [aut] (ORCID: <https://orcid.org/0000-0002-8098-9984>),
  Marta Zaffaroni [aut] (ORCID: <https://orcid.org/0000-0002-2951-8626>),
  Jean-François Rey [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3281-6701>),
  Julien Papaix [aut],
  Jean-Loup Gaussen [ctb],
  Manon Couty [ctb]
Repository: CRAN
Date/Publication: 2026-01-07 19:50:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 13:45:03 UTC; unix
Archs: landsepi.so.dSYM
