# Generated by roxygen2: do not edit by hand

export(AgriLand)
export(allocateCroptypeCultivars)
export(allocateCultivarGenes)
export(allocateLandscapeCroptypes)
export(antideriv_verhulst)
export(checkSimulParams)
export(compute_audpc100S)
export(createSimulParams)
export(demo_landsepi)
export(epid_output)
export(evol_output)
export(getMatrixCroptypePatho)
export(getMatrixCultivarPatho)
export(getMatrixGenePatho)
export(getMatrixPolyPatho)
export(inoculumToMatrix)
export(invlogit)
export(is.in.01)
export(is.positive)
export(is.strict.positive)
export(is.wholenumber)
export(loadCroptypes)
export(loadCultivar)
export(loadDispersalHost)
export(loadDispersalPathogen)
export(loadGene)
export(loadInoculum)
export(loadLandscape)
export(loadOutputs)
export(loadPathogen)
export(loadSimulParams)
export(loadTreatment)
export(logit)
export(model_landsepi)
export(multiN)
export(periodic_cov)
export(plot_allocation)
export(plot_freqPatho)
export(plotland)
export(price_reduction)
export(runShinyApp)
export(runSimul)
export(saveDeploymentStrategy)
export(setCroptypes)
export(setCultivars)
export(setDispersalHost)
export(setDispersalPathogen)
export(setGenes)
export(setInoculum)
export(setLandscape)
export(setOutputs)
export(setPathogen)
export(setSeed)
export(setSeedValue)
export(setTime)
export(setTreatment)
export(simul_landsepi)
export(survivalProbToMatrix)
export(switch_patho_to_aggr)
export(updateReproSexProb)
export(updateSurvivalProb)
export(video)
exportClasses(LandsepiParams)
exportMethods(print)
exportMethods(show)
exportMethods(summary)
import(DBI)
import(RSQLite)
import(Rcpp)
import(fields)
import(graphics)
import(methods)
import(sf)
import(sp)
import(stats)
importFrom(Matrix,nearPD)
importFrom(deSolve,ode)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grDevices,gray)
importFrom(grDevices,png)
importFrom(grDevices,tiff)
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,detectCores)
importFrom(sf,st_as_sf)
importFrom(sf,st_read)
importFrom(sf,st_write)
importFrom(splancs,polymap)
importFrom(utils,data)
importFrom(utils,installed.packages)
importFrom(utils,tail)
importFrom(utils,write.table)
useDynLib(landsepi, .registration = TRUE)
